/*************************************************************************************
 *
 * $RCSfile: CTTMax7219Lib.h,v $
 * $Date: 2014/07/20 02:16:39 $
 * $Revision: 1.1 $
 *
 * This is a basic library for the Max7219CNG display driver.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
 * PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 * 
 * (c)2014, Claude "Tryphon" Theroux, Montreal, Quebec, 
 * 
 ************************************************************************************/

#ifndef __CTTMAX7219
#define __CTTMAX7219

#include <Arduino.h>
  
/** Brightness Max7219 address. */  
#define ADDRESS_DECODE_MODE  0x09
#define ADDRESS_BRIGHTNESS   0x0a
#define ADDRESS_SCAN_LIMIT   0x0b
#define ADDRESS_SHUTDOWN     0x0c
#define ADDRESS_TEST_MODE    0x0f
  
class CTTMax7219
{
  public:
    
    /**
     * BCD modes.
     */
    enum BCDModes {
      NO_DECODE = 0,
      ONE_DIGIT = 1,
      FOUR_DIGITS = 15,
      EIGHT_DIGITS = 255
    };
  
  
    /**
     * Constructor.
     *
     * @param pCLK is clock pin.
     * @param pCS is chip select pin.
     * @param pDIN is Data in pin.
     */
    CTTMax7219(int pCLK, int pCS, int pDIN);
    
    /**
     * Initialize the chip.
     */
    void initialize();
    
    /**
     * Writes a byte to the Max7219 chip at a specific address.
     *
     * @param pAddress is the address to write the byte to.
     * @param pByte is the byte to write.
     */    
    void write(unsigned char pAddress, unsigned char pByte);
    
    /**
     * Sets the brigthness of the display.
     *
     * @param pBrighness from 0x00 to 0x0f.
     */
     void setBrightness(byte pBrightness);
     
     /**
      * Enables/disables test mode.
      *
      * @param pEnable is true to set test mode, false to disable it.
      */
      void test(bool pEnable); 
      
      /**
       * Shutdown.
       *
       * @param pEnable is true to shutdown, false otherwise.
       */
       void shutdown(bool pEnable = true);
      
      /**
       * Sets the BDC decoding mode.
       *
       * @param pBCDMode is one of the valide BCDMode enum.
       */
       void setBCDMode(BCDModes pBCDMode);
       
       /**
        * Sets the scan limit.
        *
        * @param pScanLimit from 1 to 8 digits.
        */
        void setScanLimit(byte pScanLimit);
         
      
  protected:
    /** Clock pin. */
    byte m_CLK;
    /** Chip select pin. */
    byte m_CS;
    /** Data in pin. */
    byte m_DIN;
    /** Brightness value. */
    byte m_Brightness = 0;
    /** BCD Mode. */
    enum BCDModes BCDMode = NO_DECODE;
    /** Scan limit: from 0 to 7 meaning from 1 to 8 digits. */
    byte scanLimit = 7;

    /**
     * Writes a byte to the Max7219 chip.
     * @param pByte is the byte to write.
     */    
    void writeByte(unsigned char pByte);
};

#endif

